<?php

/**
* Copyright Maarch since 2008 under licence GPLv3.
* See LICENCE.txt file at the root folder for more details.
* This file is part of Maarch software.
*
*/

/**
* @brief   BasketModelAbstract
* @author  <dev@maarch.org>
*/

namespace Basket\models;

use SrcCore\models\ValidatorModel;
use Resource\models\ResModel;
use SrcCore\controllers\PreparedClauseController;
use SrcCore\models\CoreConfigModel;
use SrcCore\models\DatabaseModel;
use User\models\UserBasketPreferenceModel;
use User\models\UserModel;

class BasketModelAbstract
{
    public static function get(array $aArgs = [])
    {
        ValidatorModel::arrayType($aArgs, ['select', 'where', 'data', 'orderBy']);

        $aBaskets = DatabaseModel::select([
            'select'    => empty($aArgs['select']) ? ['*'] : $aArgs['select'],
            'table'     => ['baskets'],
            'where'     => $aArgs['where'],
            'data'      => $aArgs['data'],
            'order_by'  => $aArgs['orderBy']
        ]);

        return $aBaskets;
    }

    public static function getById(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::stringType($aArgs, ['id']);
        ValidatorModel::arrayType($aArgs, ['select']);

        $aBasket = DatabaseModel::select([
            'select'    => empty($aArgs['select']) ? ['*'] : $aArgs['select'],
            'table'     => ['baskets'],
            'where'     => ['basket_id = ?'],
            'data'      => [$aArgs['id']]
        ]);

        return $aBasket[0];
    }

    public static function create(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'basket_name', 'basket_desc', 'clause', 'isVisible', 'isFolderBasket', 'flagNotif']);
        ValidatorModel::stringType($aArgs, ['id', 'basket_name', 'color', 'basket_desc', 'clause', 'isVisible', 'isFolderBasket', 'flagNotif']);

        DatabaseModel::insert([
            'table'         => 'baskets',
            'columnsValues' => [
                'basket_id'         => $aArgs['id'],
                'basket_name'       => $aArgs['basket_name'],
                'basket_desc'       => $aArgs['basket_desc'],
                'basket_clause'     => $aArgs['clause'],
                'is_visible'        => $aArgs['isVisible'],
                'is_folder_basket'  => $aArgs['isFolderBasket'],
                'flag_notif'        => $aArgs['flagNotif'],
                'color'             => $aArgs['color'],
                'coll_id'           => 'letterbox_coll',
            ]
        ]);

        return true;
    }

    public static function update(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'basket_name', 'basket_desc', 'clause', 'isVisible', 'isFolderBasket', 'flagNotif']);
        ValidatorModel::stringType($aArgs, ['id', 'basket_name', 'color', 'basket_desc', 'clause', 'isVisible', 'isFolderBasket', 'flagNotif']);

        DatabaseModel::update([
            'table'     => 'baskets',
            'set'       => [
                'basket_name'       => $aArgs['basket_name'],
                'basket_desc'       => $aArgs['basket_desc'],
                'basket_clause'     => $aArgs['clause'],
                'is_visible'        => $aArgs['isVisible'],
                'is_folder_basket'  => $aArgs['isFolderBasket'],
                'flag_notif'        => $aArgs['flagNotif'],
                'color'             => $aArgs['color'],
                'coll_id'           => 'letterbox_coll',
            ],
            'where'     => ['basket_id = ?'],
            'data'      => [$aArgs['id']]
        ]);

        return true;
    }

    public static function updateOrder(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::stringType($aArgs, ['id']);
        ValidatorModel::intVal($aArgs, ['order']);

        DatabaseModel::update([
            'table'     => 'baskets',
            'set'       => [
                'basket_order'  => $aArgs['order']
            ],
            'where'     => ['basket_id = ?'],
            'data'      => [$aArgs['id']]
        ]);

        return true;
    }

    public static function delete(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id']);
        ValidatorModel::stringType($aArgs, ['id']);

        DatabaseModel::delete([
            'table' => 'baskets',
            'where' => ['basket_id = ?'],
            'data'  => [$aArgs['id']]
        ]);
        DatabaseModel::delete([
            'table' => 'groupbasket',
            'where' => ['basket_id = ?'],
            'data'  => [$aArgs['id']]
        ]);
        DatabaseModel::delete([
            'table' => 'groupbasket_redirect',
            'where' => ['basket_id = ?'],
            'data'  => [$aArgs['id']]
        ]);
        DatabaseModel::delete([
            'table' => 'groupbasket_status',
            'where' => ['basket_id = ?'],
            'data'  => [$aArgs['id']]
        ]);
        DatabaseModel::delete([
            'table' => 'actions_groupbaskets',
            'where' => ['basket_id = ?'],
            'data'  => [$aArgs['id']]
        ]);

        return true;
    }

    public static function createGroupAction(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'groupId', 'actionId', 'usedInBasketlist', 'usedInActionPage', 'defaultActionList']);
        ValidatorModel::stringType($aArgs, ['id', 'groupId', 'whereClause', 'usedInBasketlist', 'usedInActionPage', 'defaultActionList']);
        ValidatorModel::intVal($aArgs, ['actionId']);

        DatabaseModel::insert([
            'table'         => 'actions_groupbaskets',
            'columnsValues' => [
                'id_action'             => $aArgs['actionId'],
                'where_clause'          => $aArgs['whereClause'],
                'group_id'              => $aArgs['groupId'],
                'basket_id'             => $aArgs['id'],
                'used_in_basketlist'    => $aArgs['usedInBasketlist'],
                'used_in_action_page'   => $aArgs['usedInActionPage'],
                'default_action_list'   => $aArgs['defaultActionList'],
            ]
        ]);

        return true;
    }

    public static function getGroupActionRedirect(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['select']);
        ValidatorModel::arrayType($aArgs, ['select', 'where', 'data']);

        $aRedirects = DatabaseModel::select([
            'select'    => $aArgs['select'],
            'table'     => ['groupbasket_redirect'],
            'where'     => $aArgs['where'],
            'data'      => $aArgs['data']
        ]);

        return $aRedirects;
    }

    public static function createGroupActionRedirect(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'groupId', 'actionId', 'redirectMode']);
        ValidatorModel::stringType($aArgs, ['id', 'groupId', 'entityId', 'keyword', 'redirectMode']);
        ValidatorModel::intVal($aArgs, ['actionId']);

        DatabaseModel::insert([
            'table'         => 'groupbasket_redirect',
            'columnsValues' => [
                'action_id'     => $aArgs['actionId'],
                'group_id'      => $aArgs['groupId'],
                'basket_id'     => $aArgs['id'],
                'entity_id'     => $aArgs['entityId'],
                'keyword'       => $aArgs['keyword'],
                'redirect_mode' => $aArgs['redirectMode']
            ]
        ]);

        return true;
    }

    public static function updateGroupActionRedirect(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['set', 'where', 'data']);
        ValidatorModel::arrayType($aArgs, ['set', 'where', 'data']);

        DatabaseModel::update([
            'table' => 'groupbasket_redirect',
            'set'   => $aArgs['set'],
            'where' => $aArgs['where'],
            'data'  => $aArgs['data']
        ]);

        return true;
    }

    public static function getGroupActionStatus(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['select']);
        ValidatorModel::arrayType($aArgs, ['select', 'where', 'data']);

        $aStatuses = DatabaseModel::select([
            'select'    => $aArgs['select'],
            'table'     => ['groupbasket_status'],
            'where'     => $aArgs['where'],
            'data'      => $aArgs['data']
        ]);

        return $aStatuses;
    }

    public static function createGroupActionStatus(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'groupId', 'actionId', 'statusId']);
        ValidatorModel::stringType($aArgs, ['id', 'groupId', 'statusId']);
        ValidatorModel::intVal($aArgs, ['actionId']);

        DatabaseModel::insert([
            'table'         => 'groupbasket_status',
            'columnsValues' => [
                'action_id'     => $aArgs['actionId'],
                'group_id'      => $aArgs['groupId'],
                'basket_id'     => $aArgs['id'],
                'status_id'     => $aArgs['statusId']
            ]
        ]);

        return true;
    }

    public static function getActionsForGroupById(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'groupId']);
        ValidatorModel::stringType($aArgs, ['id', 'groupId']);
        ValidatorModel::arrayType($aArgs, ['select']);

        $aGroups = DatabaseModel::select([
            'select'    => empty($aArgs['select']) ? ['*'] : $aArgs['select'],
            'table'     => ['actions_groupbaskets'],
            'where'     => ['basket_id = ?', 'group_id = ?'],
            'data'      => [$aArgs['id'], $aArgs['groupId']]
        ]);

        return $aGroups;
    }

    public static function hasGroup(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['id', 'groupId']);
        ValidatorModel::stringType($aArgs, ['id', 'groupId']);

        return !empty(GroupBasketModel::get(['where' => ['basket_id = ?', 'group_id = ?'], 'data' => [$aArgs['id'], $aArgs['groupId']]]));
    }

    public static function getResListById(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['basketId', 'userId']);
        ValidatorModel::stringType($aArgs, ['basketId', 'userId']);
        ValidatorModel::arrayType($aArgs, ['select']);

        $aBasket = DatabaseModel::select([
            'select'    => ['basket_clause', 'basket_res_order'],
            'table'     => ['baskets'],
            'where'     => ['basket_id = ?'],
            'data'      => [$aArgs['basketId']]
        ]);

        if (empty($aBasket[0]) || empty($aBasket[0]['basket_clause'])) {
            return [];
        }

        $where = PreparedClauseController::getPreparedClause(['clause' => $aBasket[0]['basket_clause'], 'userId' => $aArgs['userId']]);

        $aResList = ResModel::getOnView([
            'select'    => $aArgs['select'],
            'where'     => [$where],
            'orderBy'   => empty($aBasket[0]['basket_res_order']) ? ['creation_date DESC'] : [$aBasket[0]['basket_res_order']],
        ]);

        return $aResList;
    }

    public static function getBasketsByUserId(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId']);
        ValidatorModel::stringType($aArgs, ['userId']);
        ValidatorModel::arrayType($aArgs, ['unneededBasketId']);
        ValidatorModel::boolType($aArgs, ['absenceUneeded']);

        $userGroups = UserModel::getGroupsByUserId(['userId' => $aArgs['userId']]);
        $groupIds = [];
        foreach ($userGroups as $value) {
            $groupIds[] = $value['group_id'];
        }

        $aBaskets = [];
        if (!empty($groupIds)) {
            $where = ['groupbasket.group_id in (?)', 'groupbasket.basket_id = baskets.basket_id', 'groupbasket.group_id = usergroups.group_id'];
            $data = [$groupIds];
            if (!empty($aArgs['unneededBasketId'])) {
                $where[] = 'groupbasket.basket_id not in (?)';
                $data[]  = $aArgs['unneededBasketId'];
            }
            $aBaskets = DatabaseModel::select([
                    'select'    => ['usergroups.id as groupSerialId', 'groupbasket.basket_id', 'groupbasket.group_id', 'basket_name', 'basket_desc', 'basket_clause'],
                    'table'     => ['groupbasket, baskets, usergroups'],
                    'where'     => $where,
                    'data'      => $data,
                    'order_by'  => ['groupbasket.group_id, basket_order, basket_name']
            ]);

            $user = UserModel::getByUserId(['userId' => $aArgs['userId'], 'select' => ['id']]);
            foreach ($aBaskets as $key => $value) {
                unset($aBaskets[$key]['groupserialid']);
                $aBaskets[$key]['groupSerialId'] = $value['groupserialid'];
                $aBaskets[$key]['is_virtual'] = 'N';
                $aBaskets[$key]['basket_owner'] = $aArgs['userId'];
                $aBaskets2 = DatabaseModel::select([
                        'select'    => ['new_user'],
                        'table'     => ['user_abs'],
                        'where'     => ['user_abs = ?', 'basket_id = ?'],
                        'data'      => [$aArgs['userId'], $value['basket_id']],
                ]);
                $aBaskets[$key]['userToDisplay'] = UserModel::getLabelledUserById(['userId' => $aBaskets2[0]['new_user']]);
                $aBaskets[$key]['enabled'] = true;
                $userPref = UserBasketPreferenceModel::get([
                    'select'    => [1],
                    'where'     => ['user_serial_id = ?', 'group_serial_id = ?', 'basket_id = ?'],
                    'data'      => [$user['id'], $value['groupserialid'], $value['basket_id']]
                ]);
                $aBaskets[$key]['allowed'] = !empty($userPref);
            }
            if (empty($aArgs['absenceUneeded'])) {
                $aBaskets = array_merge($aBaskets, BasketModel::getAbsBasketsByUserId(['userId' => $aArgs['userId']]));
            }
        }

        return $aBaskets;
    }

    public static function getAbsBasketsByUserId(array $aArgs = [])
    {
        ValidatorModel::notEmpty($aArgs, ['userId']);
        ValidatorModel::stringType($aArgs, ['userId']);

        $aBaskets = DatabaseModel::select([
                'select'    => ['ba.basket_id', 'ba.basket_name', 'ua.user_abs', 'ua.basket_owner', 'ua.is_virtual'],
                'table'     => ['baskets ba, user_abs ua'],
                'where'     => ['ua.new_user = ?', 'ua.basket_id = ba.basket_id'],
                'data'      => [$aArgs['userId']],
                'order_by'  => ['ba.basket_order, ba.basket_name']
        ]);

        foreach ($aBaskets as $key => $value) {
            $aBaskets[$key]['userToDisplay'] = UserModel::getLabelledUserById(['userId' => $value['user_abs']]);
        }

        return $aBaskets;
    }

    public static function setRedirectedBaskets(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userAbs', 'newUser', 'basketId', 'basketOwner', 'isVirtual']);
        ValidatorModel::stringType($aArgs, ['userAbs', 'newUser', 'basketId', 'basketOwner', 'isVirtual']);

        DatabaseModel::insert([
            'table'         => 'user_abs',
            'columnsValues' => [
                'user_abs'      => $aArgs['userAbs'],
                'new_user'      => $aArgs['newUser'],
                'basket_id'     => $aArgs['basketId'],
                'basket_owner'  => $aArgs['basketOwner'],
                'is_virtual'    => $aArgs['isVirtual']
            ]
        ]);

        return true;
    }

    public static function updateRedirectedBaskets(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId', 'basketOwner', 'basketId', 'userAbs', 'newUser']);
        ValidatorModel::stringType($aArgs, ['userId']);

        DatabaseModel::update([
            'table'     => 'user_abs',
            'set'       => [
                'new_user' => $aArgs['newUser']
            ],
            'where'     => ['basket_id = ?', 'basket_owner = ?', 'user_abs = ?', 'new_user = ?'],
            'data'      => [$aArgs['basketId'], $aArgs['basketOwner'], $aArgs['userAbs'], $aArgs['userId']]
        ]);

        return true;
    }

    public static function deleteBasketRedirection(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId', 'basketId']);
        ValidatorModel::stringType($aArgs, ['userId', 'basketId']);

        DatabaseModel::delete([
            'table' => 'user_abs',
            'where' => ['(user_abs = ? OR basket_owner = ?)', 'basket_id = ?'],
            'data'  => [$aArgs['userId'], $aArgs['userId'], $aArgs['basketId']]
        ]);

        return true;
    }

    public static function getRedirectedBasketsByUserId(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId']);
        ValidatorModel::stringType($aArgs, ['userId']);

        $aBaskets = DatabaseModel::select([
            'select'    => ['ba.basket_id', 'ba.basket_name', 'ua.new_user', 'ua.basket_owner'],
            'table'     => ['baskets ba, user_abs ua'],
            'where'     => ['ua.user_abs = ?', 'ua.basket_id = ba.basket_id'],
            'data'      => [$aArgs['userId']],
            'order_by'  => ['ua.system_id']
        ]);

        foreach ($aBaskets as $key => $value) {
            $user = UserModel::getByUserId(['userId' => $value['new_user'], 'select' => ['firstname', 'lastname']]);
            $aBaskets[$key]['userToDisplay']     = "{$user['firstname']} {$user['lastname']}";
            $aBaskets[$key]['userIdRedirection'] = $value['new_user'];
            $aBaskets[$key]['user']              = "{$user['firstname']} {$user['lastname']}" ;
        }

        return $aBaskets;
    }

    public static function getRegroupedBasketsByUserId(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId']);
        ValidatorModel::stringType($aArgs, ['userId']);

        $regroupedBaskets = [];

        $user = UserModel::getByUserId(['userId' => $aArgs['userId'], 'select' => ['id']]);

        $groups = UserModel::getGroupsByUserId(['userId' => $aArgs['userId']]);
        foreach ($groups as $group) {
            $baskets = DatabaseModel::select([
                'select'    => ['baskets.basket_id', 'baskets.basket_name', 'baskets.color'],
                'table'     => ['groupbasket, baskets'],
                'where'     => ['groupbasket.basket_id = baskets.basket_id', 'groupbasket.group_id = ?', 'baskets.is_visible = ?', 'baskets.basket_id != ?'],
                'data'      => [$group['group_id'], 'Y', 'IndexingBasket'],
                'order_by'  => ['baskets.basket_order', 'baskets.basket_name']
            ]);
            $coloredBaskets = DatabaseModel::select([
                'select'    => ['basket_id', 'color'],
                'table'     => ['users_baskets'],
                'where'     => ['user_serial_id = ?', 'group_id = ?', 'color is not null'],
                'data'      => [$user['id'], $group['group_id']]
            ]);

            foreach ($baskets as $kBasket => $basket) {
                foreach ($coloredBaskets as $coloredBasket) {
                    if ($basket['basket_id'] == $coloredBasket['basket_id']) {
                        $baskets[$kBasket]['color'] = $coloredBasket['color'];
                    }
                }
                if (empty($baskets[$kBasket]['color'])) {
                    $baskets[$kBasket]['color'] = '#666666';
                }
            }

            $regroupedBaskets[] = [
                'groupId'     => $group['group_id'],
                'groupDesc'   => $group['group_desc'],
                'baskets'     => $baskets
            ];
        }

        return $regroupedBaskets;
    }

    public static function getColoredBasketsByUserId(array $aArgs)
    {
        ValidatorModel::notEmpty($aArgs, ['userId']);
        ValidatorModel::stringType($aArgs, ['userId']);

        $user = UserModel::getByUserId(['userId' => $aArgs['userId'], 'select' => ['id']]);

        $coloredBaskets = DatabaseModel::select([
            'select'    => ['basket_id', 'group_id', 'color'],
            'table'     => ['users_baskets'],
            'where'     => ['user_serial_id = ?', 'color is not null'],
            'data'      => [$user['id']]
        ]);

        return $coloredBaskets;
    }

    public static function getBasketPages(array $aArgs = [])
    {
        ValidatorModel::arrayType($aArgs, ['unneeded']);

        $customId = CoreConfigModel::getCustomId();
        if (file_exists("custom/{$customId}/modules/basket/xml/basketpage.xml")) {
            $path = "custom/{$customId}/modules/basket/xml/basketpage.xml";
        } else {
            $path = 'modules/basket/xml/basketpage.xml';
        }

        $basketPages = [];
        if (file_exists($path)) {
            $loadedXml = simplexml_load_file($path);
            if ($loadedXml) {
                foreach ($loadedXml->BASKETPAGE as $value) {
                    if (empty($aArgs['unneeded']) || !in_array((string)$value->ID, $aArgs['unneeded'])) {
                        $basketPages[] = [
                            'id'    => (string)$value->ID,
                            'label' => constant((string)$value->LABEL),
                            'name'  => (string)$value->NAME
                        ];
                    }
                }
            }
        }

        return $basketPages;
    }
}